@@ -1,4 +1,18 @@
 # -*- cperl -*-
+# Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 # This is a library file used by the Perl version of mysql-test-run,
 # and is part of the translation of the Bourne shell script with the
@@ -13,6 +27,24 @@ sub collect_one_test_case ($$$$$$$);
 
 sub mtr_options_from_test_file($$);
 
+my $do_test;
+my $skip_test;
+
+sub init_pattern {
+  my ($from, $what)= @_;
+  if ( $from =~ /^[a-z0-9]$/ ) {
+    # Does not contain any regex, make the pattern match
+    # beginning of string
+    $from= "^$from";
+  }
+  # Check that pattern is a valid regex
+  eval { "" =~/$from/; 1 } or
+    mtr_error("Invalid regex '$from' passed to $what\nPerl says: $@");
+  return $from;
+}
+
+
+
 ##############################################################################
 #
 #  Collect information about test cases we are to run
@@ -25,6 +57,9 @@ sub collect_test_cases ($) {
   my $testdir;
   my $resdir;
 
+  $do_test= init_pattern($::opt_do_test, "--do-test");
+  $skip_test= init_pattern($::opt_skip_test, "--skip-test");
+
   if ( $suite eq "main" )
   {
     $testdir= "$::glob_mysql_test_dir/t";
@@ -148,8 +183,7 @@ sub collect_test_cases ($) {
       }
 
       # Skip tests that does not match the --do-test= filter
-      next if $::opt_do_test and
-	! defined mtr_match_prefix($elem,$::opt_do_test);
+      next if ($do_test and not $tname =~ /$do_test/o);
 
       collect_one_test_case($testdir,$resdir,$tname,$elem,$cases,\%disabled,
         $component_id);
@@ -212,8 +246,10 @@ sub collect_test_cases ($) {
 	push(@criteria, "ndb=" . ($tinfo->{'ndb_test'} ? "1" : "0"));
 	# Group test with equal options together.
 	# Ending with "~" makes empty sort later than filled
-	push(@criteria, join("!", sort @{$tinfo->{'master_opt'}}) . "~");
-
+	if ( $tinfo->{'master_opt'}[0] )
+	{
+	  push(@criteria, join("!", sort @{$tinfo->{'master_opt'}}) . "~");
+	}
 	$sort_criteria{$test_name} = join(" ", @criteria);
       }
     }
@@ -274,7 +310,7 @@ sub collect_one_test_case($$$$$$$) {
   # Skip some tests but include in list, just mark them to skip
   # ----------------------------------------------------------------------
 
-  if ( $::opt_skip_test and defined mtr_match_prefix($tname,$::opt_skip_test) )
+  if ( $skip_test and $tname =~ /$skip_test/o )
   {
     $tinfo->{'skip'}= 1;
     return;
@@ -288,6 +324,7 @@ sub collect_one_test_case($$$$$$$) {
   $tinfo->{'timezone'}= "GMT-3"; # for UNIX_TIMESTAMP tests to work
 
   $tinfo->{'slave_num'}= 0; # Default, no slave
+  $tinfo->{'master_num'}= 1; # Default, 1 master
   if ( defined mtr_match_prefix($tname,"rpl") )
   {
     if ( $::opt_skip_rpl )
@@ -297,13 +334,8 @@ sub collect_one_test_case($$$$$$$) {
       return;
     }
 
-
     $tinfo->{'slave_num'}= 1; # Default for rpl* tests, use one slave
 
-    if ( $tname eq 'rpl_failsafe' or $tname eq 'rpl_chain_temp_table' )
-    {
-      # $tinfo->{'slave_num'}= 3;         # Not 3 ? Check old code, strange
-    }
   }
 
   if ( defined mtr_match_prefix($tname,"federated") )
@@ -344,6 +376,13 @@ sub collect_one_test_case($$$$$$$) {
 	next;
       }
 
+      $value= mtr_match_prefix($opt, "--slave-num=");
+      if ( defined $value )
+      {
+	$tinfo->{'slave_num'}= $value;
+	next;
+      }
+
       $value= mtr_match_prefix($opt, "--result-file=");
       if ( defined $value )
       {
@@ -488,6 +527,17 @@ sub collect_one_test_case($$$$$$$) {
   {
     mtr_options_from_test_file($tinfo,"$testdir/${tname}.test");
 
+    if ( defined $::used_default_engine )
+    {
+      # Different default engine is used
+      # tag test to require that engine
+      $tinfo->{'ndb_test'}= 1
+	if ( $::used_default_engine =~ /^ndb/i );
+
+      $tinfo->{'innodb_test'}= 1
+	if ( $::used_default_engine =~ /^innodb/i );
+    }
+
     if ( $tinfo->{'big_test'} and ! $::opt_big_test )
     {
       $tinfo->{'skip'}= 1;
@@ -531,7 +581,7 @@ sub collect_one_test_case($$$$$$$) {
       {
 	# Ndb is not supported, skip it
 	$tinfo->{'skip'}= 1;
-	$tinfo->{'comment'}= "No ndbcluster support";
+	$tinfo->{'comment'}= "No ndbcluster support or ndb tests disabled";
 	return;
       }
       elsif ( $::opt_skip_ndbcluster )
@@ -541,6 +591,8 @@ sub collect_one_test_case($$$$$$$) {
 	$tinfo->{'comment'}= "No ndbcluster tests(--skip-ndbcluster)";
 	return;
       }
+      # Ndb tests run with two mysqld masters
+      $tinfo->{'master_num'}= 2;
     }
     else
     {
@@ -556,7 +608,7 @@ sub collect_one_test_case($$$$$$$) {
 
     if ( $tinfo->{'innodb_test'} )
     {
-      # This is a test that need inndob
+      # This is a test that need innodb
       if ( $::mysqld_variables{'innodb'} eq "FALSE" )
       {
 	# innodb is not supported, skip it
@@ -566,6 +618,25 @@ sub collect_one_test_case($$$$$$$) {
       }
     }
 
+    if ( $tinfo->{'need_binlog'} )
+    {
+      if (grep(/^--skip-log-bin/,  @::opt_extra_mysqld_opt) )
+      {
+	$tinfo->{'skip'}= 1;
+	$tinfo->{'comment'}= "Test need binlog";
+	return;
+      }
+    }
+    else
+    {
+      if ( $::mysql_version_id >= 50100 )
+      {
+	# Test does not need binlog, add --skip-binlog to
+	# the options used when starting it
+	push(@{$tinfo->{'master_opt'}}, "--skip-log-bin");
+      }
+    }
+
   }
 }
 
@@ -578,6 +649,7 @@ our @tags=
  ["include/have_binlog_format_row.inc", "binlog_format", "row"],
  ["include/have_binlog_format_statement.inc", "binlog_format", "stmt"],
  ["include/have_binlog_format_mixed.inc", "binlog_format", "mixed"],
+ ["include/have_log_bin.inc", "need_binlog", 1],
  ["include/big_test.inc", "big_test", 1],
  ["include/have_debug.inc", "need_debug", 1],
  ["include/have_ndb.inc", "ndb_test", 1],
